﻿namespace WarmlyShip
{
    internal class DrawningWarmlyShip
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityWarmlyShip Ship { get; private set; }
        /// <summary>
        /// Левая координата отрисовки коробля
        /// </summary>
        private float _startPosX;
        /// <summary>
        /// Верхняя кооридната отрисовки коробля
        /// </summary>
        private float _startPosY;
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private int? _pictureWidth = null;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private int? _pictureHeight = null;
        /// <summary>
        /// Ширина отрисовки коробля
        /// </summary>
        private readonly int _shipWidth = 170;
        /// <summary>
        /// Высота отрисовки коробля
        /// </summary>
        private readonly int _shipHeight = 65;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес коробля</param>
        /// <param name="bodyColor">Цвет основной части</param>
        public void Init(int speed, float weight, Color bodyColor)
        {
            Ship = new EntityWarmlyShip();
            Ship.Init(speed, weight, bodyColor);
        }                       
        /// <summary>
        /// Установка позиции коробля
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public void SetPosition(int x, int y, int width, int height)
        {
            if (x < 0 || y < 0 || width < x + _shipWidth || height < y + _shipHeight)
            {
                _pictureHeight = null;
                _pictureWidth = null;
                return;
            }
            _startPosX = x;
            _startPosY = y;
            _pictureWidth = width;
            _pictureHeight = height;
        }
        /// <summary>
        /// Изменение направления пермещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(Direction direction)
        {
            if (!_pictureWidth.HasValue || !_pictureHeight.HasValue)
            {
                return;
            }
            switch (direction)
            {
                // вправо
                case Direction.Right:
                    if (_startPosX + _shipWidth + Ship.Step < _pictureWidth)
                    {
                        _startPosX += Ship.Step;
                    }
                    break;
                //влево
                case Direction.Left:
                    
                    if (_startPosX - Ship.Step > 0)
                    {
                        _startPosX -= Ship.Step;
                    }
                    break;
                //вверх
                case Direction.Up:
                    
                    if (_startPosY - Ship.Step > 0)
                    {
                        _startPosY -= Ship.Step;
                    }
                    break;
                //вниз
                case Direction.Down:
                    if (_startPosY + _shipHeight + Ship.Step < _pictureHeight)
                    {
                        _startPosY += Ship.Step;
                    }
                    break;
            }
        }
        /// <summary>
        /// Отрисовка корабля
        /// </summary>
        /// <param name="g"></param>
        public void DrawTransport(Graphics g)
        {
            if (_startPosX < 0 || _startPosY < 0
                || !_pictureHeight.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }
            

            //заполнение коробля

            PointF[] P =
                {
                    new Point((int)(_startPosX),(int) _startPosY + 45),
                    new Point((int)_startPosX + 170, (int)_startPosY + 45),
                    new Point( (int) _startPosX + 150,(int) _startPosY + 65),
                    new Point((int) _startPosX + 50, (int) _startPosY + 65),
                    new Point((int) _startPosX, (int) _startPosY + 45),

            };
            Brush br = new SolidBrush(Ship?.BodyColor ?? Color.Black);
            g.FillPolygon(br, P);

            Brush brBlue = new SolidBrush(Color.LightBlue);
            g.FillRectangle(brBlue, _startPosX + 70, _startPosY + 20, 70, 25);

            Pen pen = new(Color.Black);
            //границы коробля
            g.DrawRectangle(pen, _startPosX + 70, _startPosY + 20 , 70, 25);
            g.DrawLine(pen, _startPosX, _startPosY + 45, _startPosX + 170, _startPosY + 45);
            g.DrawLine(pen, _startPosX, _startPosY + 45, _startPosX + 50, _startPosY + 65);
            g.DrawLine(pen, _startPosX + 50, _startPosY + 65, _startPosX + 150, _startPosY + 65);
            g.DrawLine(pen, _startPosX + 150, _startPosY + 65, _startPosX + 170, _startPosY + 45);
        }

        /// <summary>
        /// Смена границ формы отрисовки
        /// </summary>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public void ChangeBorders(int width, int height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_pictureWidth <= _shipWidth || _pictureHeight <= _shipHeight)
            {
                _pictureWidth = null;
                _pictureHeight = null;
                return;
            }
            if (_startPosX + _shipWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth.Value - _shipWidth;
            }
            if (_startPosY + _shipHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight.Value - _shipHeight;
            }
        }
    }
}
